@@ -16,6 +16,8 @@
 
 from oslo_config import cfg
 
+from networking_arista._i18n import _
+
 
 # Arista ML2 Mechanism driver specific configuration knobs.
 #
@@ -56,10 +58,10 @@ ARISTA_DRIVER_OPTS = [
                       'field. If not set, a value of 30 seconds is '
                       'assumed.')),
     cfg.IntOpt('conn_timeout',
-               default=10,
+               default=60,
                help=_('Connection timeout interval in seconds. This interval '
-                      'defines how long an EAPI request from the driver to '
-                      'EOS waits before timing out. If not set, a value of 10 '
+                      'defines how long an API request from the driver to '
+                      'CVX waits before timing out. If not set, a value of 60 '
                       'seconds is assumed.')),
     cfg.StrOpt('region_name',
                default='RegionOne',
@@ -70,7 +72,46 @@ ARISTA_DRIVER_OPTS = [
                       'with the region name registered (or known) to keystone '
                       'service. Authentication with Keysotne is performed by '
                       'EOS. This is optional. If not set, a value of '
-                      '"RegionOne" is assumed.'))
+                      '"RegionOne" is assumed.')),
+    cfg.BoolOpt('sec_group_support',
+                default=False,
+                help=_('Specifies if the Security Groups needs to deployed '
+                       'for baremetal deployments. If this flag is set to '
+                       'True, this means switch_info(see below) must be '
+                       'defined. If this flag is not defined, it is assumed '
+                       'to be False')),
+    cfg.ListOpt('switch_info',
+                default=[],
+                help=_('This is a comma separated list of Arista switches '
+                       'where security groups (i.e. ACLs) need to be '
+                       'applied. Each string has three values separated  '
+                       'by : in the follow format '
+                       '<IP of switch>:<username>:<password>, ...... '
+                       'For Example: 172.13.23.55:admin:admin, '
+                       '172.13.23.56:admin:admin, .... '
+                       'This is required if sec_group_support is set to '
+                       '"True"')),
+    cfg.StrOpt('api_type',
+               default='EAPI',
+               help=_('Tells the plugin to use a sepcific API interfaces '
+                      'to communicate with CVX. Valid options are:'
+                      'EAPI - Use EOS\' extensible API.'
+                      'JSON - Use EOS\' JSON/REST API.')),
+    cfg.ListOpt('managed_physnets',
+                default=[],
+                help=_('This is a comma separated list of physical networks '
+                       'which are managed by Arista switches.'
+                       'This list will be used by the Arista ML2 plugin'
+                       'to make the decision if it can participate in binding'
+                       'or updating a port.'
+                       'For Example: '
+                       'managed_physnets = arista_network')),
+    cfg.BoolOpt('manage_fabric',
+                default=False,
+                help=_('Specifies whether the Arista ML2 plugin should bind '
+                       'ports to vxlan fabric segments and dynamically '
+                       'allocate vlan segments based on the host to connect '
+                       'the port to the vxlan fabric')),
 ]
 
 
