@@ -16,15 +16,15 @@ import hashlib
 import socket
 import struct
 
-import jsonrpclib
+import jsonrpc_requests
 from oslo_config import cfg
 from oslo_log import log as logging
 
-from neutron._i18n import _LI
-
+from networking_arista._i18n import _, _LI
 from networking_arista.common import exceptions as arista_exc
 
 LOG = logging.getLogger(__name__)
+cfg.CONF.import_group('l3_arista', 'networking_arista.common.config')
 
 EOS_UNREACHABLE_MSG = _('Unable to reach EOS')
 DEFAULT_VLAN = 1
@@ -35,12 +35,11 @@ IPV6_BITS = 128
 
 # This string-format-at-a-distance confuses pylint :(
 # pylint: disable=too-many-format-args
-router_in_vrf = {
+router_in_vrf_v1 = {
     'router': {'create': ['vrf definition {0}',
                           'rd {1}',
                           'exit'],
                'delete': ['no vrf definition {0}']},
-
     'interface': {'add': ['ip routing vrf {1}',
                           'vlan {0}',
                           'exit',
@@ -49,6 +48,19 @@ router_in_vrf = {
                           'ip address {2}'],
                   'remove': ['no interface vlan {0}']}}
 
+router_in_vrf_v2 = {
+    'router': {'create': ['vrf instance {0}',
+                          'rd {1}',
+                          'exit'],
+               'delete': ['no vrf instance {0}']},
+    'interface': {'add': ['ip routing vrf {1}',
+                          'vlan {0}',
+                          'exit',
+                          'interface vlan {0}',
+                          'vrf {1}',
+                          'ip address {2}'],
+                  'remove': ['no interface vlan {0}']}}
+
 router_in_default_vrf = {
     'router': {'create': [],   # Place holder for now.
                'delete': []},  # Place holder for now.
@@ -101,19 +113,32 @@ class AristaL3Driver(object):
         self._validate_config()
         host = cfg.CONF.l3_arista.primary_l3_host
         self._hosts.append(host)
-        self._servers.append(jsonrpclib.Server(self._eapi_host_url(host)))
+        self._servers.append(
+            jsonrpc_requests.Server(
+                self._eapi_host_url(host),
+                verify=False
+            )
+        )
         self._mlag_configured = cfg.CONF.l3_arista.mlag_config
+        self._vrf_syntax_v2_supported = None
+        self._router_in_vrf = router_in_vrf_v2
         self._use_vrf = cfg.CONF.l3_arista.use_vrf
         if self._mlag_configured:
             host = cfg.CONF.l3_arista.secondary_l3_host
             self._hosts.append(host)
-            self._servers.append(jsonrpclib.Server(self._eapi_host_url(host)))
+            self._servers.append(
+                jsonrpc_requests.Server(
+                    self._eapi_host_url(host),
+                    verify=False
+                )
+            )
             self._additionalRouterCmdsDict = additional_cmds_for_mlag['router']
             self._additionalInterfaceCmdsDict = (
                 additional_cmds_for_mlag['interface'])
         if self._use_vrf:
-            self.routerDict = router_in_vrf['router']
-            self._interfaceDict = router_in_vrf['interface']
+            self.routerDict = self._router_in_vrf['router']
+            self._update_vrf_commands()
+            self._interfaceDict = self._router_in_vrf['interface']
         else:
             self.routerDict = router_in_default_vrf['router']
             self._interfaceDict = router_in_default_vrf['interface']
@@ -126,6 +151,45 @@ class AristaL3Driver(object):
                            (user, pwd, host))
         return eapi_server_url
 
+    def _supports_vrf_instance(self, version):
+        ver_tokens = version.split('.')
+        if len(ver_tokens) < 2 or int(ver_tokens[0]) < 4:
+            return False
+        if int(ver_tokens[0]) == 4 and int(ver_tokens[1]) < 22:
+            return False
+        return True
+
+    def _check_vrf_syntax_v2_support(self, server):
+        cmds = ['show version']
+        result = None
+        try:
+            result = server.runCmds(version=1, cmds=cmds)
+        except Exception:
+            return False
+
+        return result and self._supports_vrf_instance(result[0].get('version',
+                                                                    ''))
+
+    def _update_vrf_commands(self):
+        # This assumes all switches run the same version. This needs to be
+        # updated if we'll support distributed routing
+        new_vrf_support = self._check_vrf_syntax_v2_support(
+            self._servers[0])
+
+        if new_vrf_support == self._vrf_syntax_v2_supported:
+            return
+
+        LOG.info(_LI('Updating VRF command supported: %s'),
+                 'vrf instance' if new_vrf_support else 'vrf definition')
+        self._vrf_syntax_v2_supported = new_vrf_support
+        if self._vrf_syntax_v2_supported is False:
+            self._router_in_vrf = router_in_vrf_v1
+        else:
+            self._router_in_vrf = router_in_vrf_v2
+        # we don't need to update self.interfaceDict as it is updated by
+        # _select_dicts function before it is used
+        self.routerDict = self._router_in_vrf['router']
+
     def _validate_config(self):
         if cfg.CONF.l3_arista.get('primary_l3_host') == '':
             msg = _('Required option primary_l3_host is not set')
@@ -183,10 +247,14 @@ class AristaL3Driver(object):
 
     def _select_dicts(self, ipv):
         if self._use_vrf:
-            self._interfaceDict = router_in_vrf['interface']
+            if ipv == 6:
+                msg = (_('IPv6 subnets are not supported with VRFs'))
+                LOG.exception(msg)
+                raise arista_exc.AristaServicePluginRpcError(msg=msg)
+            self._interfaceDict = self._router_in_vrf['interface']
         else:
             if ipv == 6:
-                # for IPv6 use IPv6 commmands
+                # for IPv6 use IPv6 commands
                 self._interfaceDict = router_in_default_vrf_v6['interface']
                 self._additionalInterfaceCmdsDict = (
                     additional_cmds_for_mlag_v6['interface'])
@@ -248,7 +316,9 @@ class AristaL3Driver(object):
         if router:
             router_name = self._arista_router_name(tenant_id, router['name'])
 
-            rdm = str(int(hashlib.sha256(router_name).hexdigest(), 16) % 65536)
+            hashed = hashlib.sha256(router_name.encode('utf-8'))
+            rdm = str(int(hashed.hexdigest(), 16) % 65536)
+
             mlag_peer_failed = False
             for s in self._servers:
                 try:
@@ -358,7 +428,7 @@ class AristaL3Driver(object):
                         LOG.exception(msg)
                         raise arista_exc.AristaServicePluginRpcError(msg=msg)
 
-    def _run_openstack_l3_cmds(self, commands, server):
+    def _run_openstack_l3_cmds(self, commands, server, log_exception=True):
         """Execute/sends a CAPI (Command API) command to EOS.
 
         In this method, list of commands is appended with prefix and
@@ -383,7 +453,13 @@ class AristaL3Driver(object):
             msg = (_('Error occurred while trying to execute '
                      'commands %(cmd)s on EOS %(host)s') %
                    {'cmd': full_command, 'host': server})
-            LOG.exception(msg)
+            if log_exception:
+                LOG.exception(msg)
+            if self._use_vrf:
+                # For now we assume that the only command that raises this
+                # exception is vrf instance/definition and we need to update
+                # the current support
+                self._update_vrf_commands()
             raise arista_exc.AristaServicePluginRpcError(msg=msg)
 
     def _arista_router_name(self, tenant_id, name):
@@ -393,7 +469,7 @@ class AristaL3Driver(object):
         can be distinguishged from the user created routers/SVIs
         on Arista HW.
         """
-        return 'OS' + '-' + tenant_id + '-' + name
+        return 'OS' + '-' + tenant_id + '-' + name.replace(' ', '_')
 
     def _get_binary_from_ipv4(self, ip_addr):
         """Converts IPv4 address to binary form."""
