@@ -15,18 +15,18 @@
 
 import threading
 
+from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as n_const
+from oslo_concurrency import lockutils
 from oslo_config import cfg
 from oslo_log import log as logging
 
 from neutron.common import constants as neutron_const
-from neutron.extensions import portbindings
 from neutron.plugins.common import constants as p_const
 from neutron.plugins.ml2.common import exceptions as ml2_exc
 from neutron.plugins.ml2 import driver_api
 
 from networking_arista._i18n import _, _LI, _LE
-from networking_arista.common import config  # noqa
 from networking_arista.common import db
 from networking_arista.common import db_lib
 from networking_arista.common import exceptions as arista_exc
@@ -34,6 +34,7 @@ from networking_arista.ml2 import arista_ml2
 from networking_arista.ml2 import sec_group_callback
 
 LOG = logging.getLogger(__name__)
+cfg.CONF.import_group('ml2_arista', 'networking_arista.common.config')
 
 # Messages
 EOS_UNREACHABLE_MSG = _('Unable to reach EOS')
@@ -72,11 +73,12 @@ class AristaDriver(driver_api.MechanismDriver):
         self.timer = None
         self.sync_timeout = confg['sync_interval']
         self.managed_physnets = confg['managed_physnets']
+        self.manage_fabric = confg['manage_fabric']
         self.eos_sync_lock = threading.Lock()
 
         self.eapi = None
 
-        if rpc:
+        if rpc is not None:
             LOG.info("Using passed in parameter for RPC")
             self.rpc = rpc
             self.eapi = rpc
@@ -208,12 +210,10 @@ class AristaDriver(driver_api.MechanismDriver):
         with self.eos_sync_lock:
             if db_lib.is_network_provisioned(tenant_id, network_id):
                 if db_lib.are_ports_attached_to_network(network_id):
-                    LOG.info(_LI('Network %s can not be deleted as it '
-                                 'has ports attached to it'), network_id)
-                    raise ml2_exc.MechanismDriverError(
-                        method='delete_network_precommit')
-                else:
-                    db_lib.forget_network_segment(tenant_id, network_id)
+                    db_lib.forget_all_ports_for_network(network_id)
+                    LOG.info(_LI('Deleting all ports on network %s'),
+                             network_id)
+                db_lib.forget_network_segment(tenant_id, network_id)
 
     def delete_network_postcommit(self, context):
         """Send network delete request to Arista HW."""
@@ -222,9 +222,9 @@ class AristaDriver(driver_api.MechanismDriver):
         if not self.rpc.hpb_supported():
             # Hierarchical port binding is not supported by CVX, only
             # send the request if network type is VLAN.
-            if(segments and
+            if (segments and
                     segments[0][driver_api.NETWORK_TYPE] != p_const.TYPE_VLAN):
-                # If networtk type is not VLAN, do nothing
+                # If network type is not VLAN, do nothing
                 return
             # No need to pass segments info when calling delete_network as
             # HPB is not supported.
@@ -272,9 +272,10 @@ class AristaDriver(driver_api.MechanismDriver):
         mac_to_hostname = physnet_info.get('mac_to_hostname', {})
         for link in link_info:
             if link.get('switch_id') in mac_to_hostname:
-                return mac_to_hostname.get(link.get('switch_id'))
+                physnet = mac_to_hostname.get(link.get('switch_id'))
+                return self.rpc.mlag_pairs.get(physnet, physnet)
 
-    def _bind_port_to_baremetal(self, context, segment, physnet_info):
+    def _bind_port_to_baremetal(self, context, segment):
 
         port = context.current
         vnic_type = port.get('binding:vnic_type')
@@ -290,21 +291,6 @@ class AristaDriver(driver_api.MechanismDriver):
         if not link_info:
             return
 
-        switch_id = physnet_info.get('switch_id')
-        mac_to_hostname = physnet_info.get('mac_to_hostname', {})
-        switch_list = []
-        for link in link_info:
-            if ((switch_id and switch_id != link.get('switch_id')) or
-                    (link.get('switch_id') not in mac_to_hostname)):
-                LOG.debug("Port %(port)s with %(link)s is not managed"
-                          " by Arista mechanism driver ", {'port': port,
-                                                           'link': link_info})
-            else:
-                switch_list.append(link.get('switch_id'))
-
-        if not switch_list:
-            return
-
         vif_details = {
             portbindings.VIF_DETAILS_VLAN: str(
                 segment[driver_api.SEGMENTATION_ID])
@@ -327,31 +313,33 @@ class AristaDriver(driver_api.MechanismDriver):
         """
         host_id = context.host
         port = context.current
-        physnet_info = self.eapi.get_physical_network(host_id)
-        physnet = physnet_info.get('physnet')
-        switch_id = physnet_info.get('switch_id')
-        if not physnet or not switch_id:
-            if port.get('binding:vnic_type') == portbindings.VNIC_BAREMETAL:
-                # Find physnet using link_local_information in baremetal case
-                physnet = self._get_physnet_from_link_info(port, physnet_info)
-            else:
-                LOG.debug("The host %(host)s not connected to arista "
-                          "switches. Physical Network info = %(pi)s",
-                          {'host': host_id, 'pi': physnet_info})
-                return
-
-        if not physnet or not self._is_in_managed_physnets(physnet):
-            LOG.debug("bind_port for port %(port)s: physical_network "
-                      "%(physnet)s is not managed by Arista "
-                      "mechanism driver", {'port': port.get('id'),
-                                           'physnet': physnet})
-            return
-
-        LOG.debug("bind_port for port %(port)s: physical_network=%(physnet)s,"
-                  "switch_id=%(swid)s", {'port': port.get('id'),
-                                         'physnet': physnet,
-                                         'swid': switch_id})
+        physnet_info = {}
         for segment in context.segments_to_bind:
+            physnet = segment.get(driver_api.PHYSICAL_NETWORK)
+            if not self._is_in_managed_physnets(physnet):
+                LOG.debug("bind_port for port %(port)s: physical_network "
+                          "%(physnet)s is not managed by Arista "
+                          "mechanism driver", {'port': port.get('id'),
+                                               'physnet': physnet})
+                continue
+            # If physnet is not set, we need to look it up using hostname
+            # and topology info
+            if not physnet:
+                if not physnet_info:
+                    # We only need to get physnet_info once
+                    physnet_info = self.eapi.get_physical_network(host_id)
+                if (port.get('binding:vnic_type') ==
+                        portbindings.VNIC_BAREMETAL):
+                    # Find physnet using link_information in baremetal case
+                    physnet = self._get_physnet_from_link_info(port,
+                                                               physnet_info)
+                else:
+                    physnet = physnet_info.get('physnet')
+            # If physnet was not found, we cannot bind this port
+            if not physnet:
+                LOG.debug("bind_port for port %(port)s: no physical_network "
+                          "found", {'port': port.get('id')})
+                continue
             if segment[driver_api.NETWORK_TYPE] == p_const.TYPE_VXLAN:
                 # Check if CVX supports HPB
                 if not self.rpc.hpb_supported():
@@ -362,10 +350,11 @@ class AristaDriver(driver_api.MechanismDriver):
                 # allocate dynamic segmentation id to bind the port to
                 # the network that the port belongs to.
                 try:
-                    next_segment = context.allocate_dynamic_segment(
-                        {'id': context.network.current['id'],
-                         'network_type': p_const.TYPE_VLAN,
-                         'physical_network': physnet})
+                    with lockutils.lock(physnet, external=True):
+                        next_segment = context.allocate_dynamic_segment(
+                            {'id': context.network.current['id'],
+                             'network_type': n_const.TYPE_VLAN,
+                             'physical_network': physnet})
                 except Exception as exc:
                     LOG.error(_LE("bind_port for port %(port)s: Failed to "
                                   "allocate dynamic segment for physnet "
@@ -380,9 +369,9 @@ class AristaDriver(driver_api.MechanismDriver):
                           {'port': port.get('id'), 'current_seg': segment,
                            'next_seg': next_segment})
                 context.continue_binding(segment['id'], [next_segment])
-            else:
+            elif port.get('binding:vnic_type') == portbindings.VNIC_BAREMETAL:
                 # The network_type is vlan, try binding process for baremetal.
-                self._bind_port_to_baremetal(context, segment, physnet_info)
+                self._bind_port_to_baremetal(context, segment)
 
     def create_port_postcommit(self, context):
         """Plug a physical host into a network.
@@ -398,6 +387,18 @@ class AristaDriver(driver_api.MechanismDriver):
 
         return
 
+    def _supported_device_owner(self, device_owner):
+        supported_device_owner = [n_const.DEVICE_OWNER_DHCP,
+                                  n_const.DEVICE_OWNER_DVR_INTERFACE]
+
+        if any([device_owner in supported_device_owner,
+                device_owner.startswith('compute') and
+                device_owner != 'compute:probe',
+                device_owner.startswith('baremetal')]):
+            return True
+
+        LOG.debug('Unsupported device owner: %s', device_owner)
+
     def _network_owner_tenant(self, context, network_id, tenant_id):
         tid = tenant_id
         if network_id and tenant_id:
@@ -409,8 +410,11 @@ class AristaDriver(driver_api.MechanismDriver):
         return tid
 
     def _is_in_managed_physnets(self, physnet):
+        # Check if this is a fabric segment
+        if not physnet:
+            return self.manage_fabric
+        # If managed physnet is empty, accept all.
         if not self.managed_physnets:
-            # If managed physnet is empty, accept all.
             return True
         # managed physnet is not empty, find for matching physnet
         return any(pn == physnet for pn in self.managed_physnets)
@@ -422,15 +426,21 @@ class AristaDriver(driver_api.MechanismDriver):
         segment is included in the managed physical network list.
         """
         if not self.managed_physnets:
-            return (context.bottom_bound_segment
-                    if context.binding_levels else None)
+            return [
+                binding_level.get(driver_api.BOUND_SEGMENT)
+                for binding_level in (context.binding_levels or
+                                      context.original_binding_levels or [])
+            ]
 
-        for binding_level in (context.binding_levels or []):
+        bound_segments = []
+        for binding_level in (context.binding_levels or
+                              context.original_binding_levels or []):
             bound_segment = binding_level.get(driver_api.BOUND_SEGMENT)
             if (bound_segment and
                 self._is_in_managed_physnets(
                     bound_segment.get(driver_api.PHYSICAL_NETWORK))):
-                return bound_segment
+                bound_segments.append(bound_segment)
+        return bound_segments
 
     def _handle_port_migration_precommit(self, context):
         """Handles port migration in precommit
@@ -439,16 +449,43 @@ class AristaDriver(driver_api.MechanismDriver):
         """
         orig_port = context.original
         orig_host = context.original_host
-        orig_status = context.original_status
-        new_status = context.status
         new_host = context.host
         new_port = context.current
         port_id = orig_port['id']
+        new_status = context.status
+        orig_status = context.original_status
 
-        if (new_host != orig_host and
-            orig_status == n_const.PORT_STATUS_ACTIVE and
-                new_status == n_const.PORT_STATUS_DOWN):
-            LOG.debug("Handling port migration for: %s " % orig_port)
+        # Deleting of a baremetal port is handled here as update_port_pre/post
+        # commit are invoked with port with the following settings:
+        # +---------------------------------------------------------------+
+        # | port attr          | current     | original                   |
+        # +====================+=============+============================+
+        # | host_id            | ""          | <uuid of baremetal>        |
+        # +---------------------+------------+----------------------------+
+        # | status             | "DOWN"      | "ACTIVE"                   |
+        # +--------------------+-------------+----------------------------+
+        # | binding:vif_type   | "unbound"   | "other"                    |
+        # +--------------------+-------------+----------------------------+
+        # | binding:vnic_type  | "baremetal" | "baremetal"                |
+        # +--------------------+-----+-------+----------------------------+
+        # | binding:           |  {}         | {"vlan": "<vlan_id>"}      |
+        # | vif_details        |             |                            |
+        # +--------------------+-------------+----------------------------+
+        # | binding:profile    |  {}         | {"local_link_information": |
+        # |                    |             |                     [...]} |
+        # +---------------------------------------------------------------+
+        # Because of the change in host_id, binding_levels in port context
+        # the database will be cleared in _process_port_binding, and when
+        # delete_port_pre/postcommit are called the required information such
+        # as host_id, binding_levels and binding:profile will not be available
+        # to remove deleted port and segment from EOS.
+        is_port_deleted = (new_status == n_const.PORT_STATUS_DOWN and
+                           orig_status == n_const.PORT_STATUS_ACTIVE and
+                           new_host != orig_host)
+        is_port_migrated = (new_host and orig_host and new_host != orig_host)
+        if is_port_deleted or is_port_migrated:
+            LOG.debug("Handling port %s for: %s " % (
+                "migration" if is_port_migrated else "delete", orig_port))
             network_id = orig_port['network_id']
             tenant_id = orig_port['tenant_id'] or INTERNAL_TENANT_ID
             # Ensure that we use tenant Id for the network owner
@@ -473,14 +510,15 @@ class AristaDriver(driver_api.MechanismDriver):
         """
         orig_port = context.original
         orig_host = context.original_host
-        orig_status = context.original_status
-        new_status = context.status
         new_host = context.host
+        new_status = context.status
+        orig_status = context.original_status
 
-        if (new_host != orig_host and
-            orig_status == n_const.PORT_STATUS_ACTIVE and
-                new_status == n_const.PORT_STATUS_DOWN):
-
+        is_port_deleted = (new_status == n_const.PORT_STATUS_DOWN and
+                           orig_status == n_const.PORT_STATUS_ACTIVE and
+                           new_host != orig_host)
+        is_port_migrated = (new_host and orig_host and new_host != orig_host)
+        if is_port_deleted or is_port_migrated:
             self._try_to_release_dynamic_segment(context, migration=True)
 
             # Handling migration case.
@@ -492,18 +530,21 @@ class AristaDriver(driver_api.MechanismDriver):
             # Ensure that we use tenant Id for the network owner
             tenant_id = self._network_owner_tenant(context, network_id,
                                                    tenant_id)
-            for binding_level in context._original_binding_levels:
+            with self.eos_sync_lock:
+                # Removing the port form original host
+                self._delete_port(context, orig_port, orig_host, tenant_id)
+
+            for binding_level in context._original_binding_levels or []:
                 if self._network_provisioned(
-                    tenant_id, network_id,
+                        context,
+                        tenant_id,
+                        network_id,
                         segment_id=binding_level.segment_id):
                     with self.eos_sync_lock:
-                        # Removing the port form original host
-                        self._delete_port(orig_port, orig_host, tenant_id)
-
                         # If segment id is not bound to any port, then
                         # remove it from EOS
                         segment = self.ndb.get_segment_by_id(
-                            context._plugin_context.session,
+                            context._plugin_context,
                             binding_level.segment_id)
                         if not segment:
                             try:
@@ -543,7 +584,7 @@ class AristaDriver(driver_api.MechanismDriver):
         pretty_log("update_port_precommit: new", new_port)
         pretty_log("update_port_precommit: orig", orig_port)
 
-        if new_port['device_owner'] == 'compute:probe':
+        if not self._supported_device_owner(new_port['device_owner']):
             return
 
         # Check if it is port migration case
@@ -555,6 +596,8 @@ class AristaDriver(driver_api.MechanismDriver):
         if not seg_info:
             # Ignoring the update as the port is not managed by
             # arista mechanism driver.
+            LOG.debug("Ignoring the update as the port is not managed by "
+                      "Arista switches.")
             return
 
         # device_id and device_owner are set on VM boot
@@ -564,23 +607,18 @@ class AristaDriver(driver_api.MechanismDriver):
         # Ensure that we use tenant Id for the network owner
         tenant_id = self._network_owner_tenant(context, network_id, tenant_id)
 
-        if not self._network_provisioned(tenant_id, network_id,
-                                         seg_info[driver_api.SEGMENTATION_ID],
-                                         seg_info[driver_api.ID]):
-            if not self.rpc.hpb_supported():
-                LOG.info(_LI("Ignoring port %(port)s conntected to "
-                             "%(net_id)s"), {'port': port_id,
-                                             'net_id': network_id})
-                return
-
-            LOG.info(
-                _LI("Adding %s to provisioned network database"), seg_info)
-            with self.eos_sync_lock:
-                db_lib.remember_tenant(tenant_id)
-                db_lib.remember_network_segment(
-                    tenant_id, network_id,
-                    seg_info[driver_api.SEGMENTATION_ID],
-                    seg_info[driver_api.ID])
+        for seg in seg_info:
+            if not self._network_provisioned(context, tenant_id, network_id,
+                                             seg[driver_api.SEGMENTATION_ID],
+                                             seg[driver_api.ID]):
+                LOG.info(
+                    _LI("Adding %s to provisioned network database"), seg)
+                with self.eos_sync_lock:
+                    db_lib.remember_tenant(tenant_id)
+                    db_lib.remember_network_segment(
+                        tenant_id, network_id,
+                        seg[driver_api.SEGMENTATION_ID],
+                        seg[driver_api.ID])
 
         with self.eos_sync_lock:
             port_down = False
@@ -647,11 +685,14 @@ class AristaDriver(driver_api.MechanismDriver):
         host = context.host
         is_vm_boot = device_id and device_owner
 
+        if not self._supported_device_owner(device_owner):
+            return
+
         vnic_type = port['binding:vnic_type']
         binding_profile = port['binding:profile']
         bindings = []
         if binding_profile:
-            bindings = binding_profile['local_link_information']
+            bindings = binding_profile.get('local_link_information', [])
 
         port_id = port['id']
         port_name = port['name']
@@ -678,7 +719,6 @@ class AristaDriver(driver_api.MechanismDriver):
 
         with self.eos_sync_lock:
             hostname = self._host_name(host)
-            segmentation_id = seg_info[driver_api.SEGMENTATION_ID]
             port_host_filter = None
             if(port['device_owner'] ==
                n_const.DEVICE_OWNER_DVR_INTERFACE):
@@ -691,16 +731,22 @@ class AristaDriver(driver_api.MechanismDriver):
             # If network does not exist under this tenant,
             # it may be a shared network. Get shared network owner Id
             net_provisioned = self._network_provisioned(
-                tenant_id, network_id, segmentation_id=segmentation_id)
+                context,
+                tenant_id,
+                network_id
+            )
+            for seg in seg_info:
+                if not self._network_provisioned(
+                    context,
+                    tenant_id,
+                    network_id,
+                    segmentation_id=seg[driver_api.SEGMENTATION_ID]):
+                    net_provisioned = False
             segments = []
             if net_provisioned and self.rpc.hpb_supported():
-                for binding_level in context.binding_levels:
-                    bound_segment = binding_level.get(
-                        driver_api.BOUND_SEGMENT)
-                    if bound_segment:
-                        segments.append(bound_segment)
+                segments = seg_info
                 all_segments = self.ndb.get_all_network_segments(
-                    network_id, session=context._plugin_context.session)
+                    network_id, context=context._plugin_context)
                 try:
                     self.rpc.create_network_segments(
                         tenant_id, network_id,
@@ -724,7 +770,12 @@ class AristaDriver(driver_api.MechanismDriver):
                         # The port moved to a different host or the VM
                         # connected to the port was deleted or its in DOWN
                         # state. So delete the old port on the old host.
-                        self._delete_port(orig_port, orig_host, tenant_id)
+                        self._delete_port(
+                            context,
+                            orig_port,
+                            orig_host,
+                            tenant_id
+                        )
                     except ml2_exc.MechanismDriverError:
                         # If deleting a port fails, then not much can be done
                         # about it. Log a warning and move on.
@@ -762,7 +813,7 @@ class AristaDriver(driver_api.MechanismDriver):
         port_id = port['id']
         host_id = context.host
         with self.eos_sync_lock:
-            if db_lib.is_port_provisioned(port_id):
+            if db_lib.is_port_provisioned(port_id, host_id):
                 db_lib.forget_port(port_id, host_id)
 
     def delete_port_postcommit(self, context):
@@ -782,20 +833,20 @@ class AristaDriver(driver_api.MechanismDriver):
         pretty_log("delete_port_postcommit:", port)
 
         # If this port is the last one using dynamic segmentation id,
-        # and the segmentaion id was alloated by this driver, it needs
-        # to be releaed.
+        # and the segmentation id was allocated by this driver, it needs
+        # to be released.
         self._try_to_release_dynamic_segment(context)
 
         with self.eos_sync_lock:
             try:
-                self._delete_port(port, host, tenant_id)
+                self._delete_port(context, port, host, tenant_id)
                 self._delete_segment(context, tenant_id)
             except ml2_exc.MechanismDriverError:
                 # Can't do much if deleting a port failed.
                 # Log a warning and continue.
                 LOG.warning(UNABLE_TO_DELETE_PORT_MSG)
 
-    def _delete_port(self, port, host, tenant_id):
+    def _delete_port(self, context, port, host, tenant_id):
         """Deletes the port from EOS.
 
         param port: Port which is to be deleted
@@ -808,11 +859,15 @@ class AristaDriver(driver_api.MechanismDriver):
         port_id = port['id']
         network_id = port['network_id']
         device_owner = port['device_owner']
+
+        if not self._supported_device_owner(device_owner):
+            return
+
         vnic_type = port['binding:vnic_type']
         binding_profile = port['binding:profile']
         switch_bindings = []
         if binding_profile:
-            switch_bindings = binding_profile['local_link_information']
+            switch_bindings = binding_profile.get('local_link_information', [])
         sg = port['security_groups']
 
         if not device_id or not host:
@@ -820,9 +875,6 @@ class AristaDriver(driver_api.MechanismDriver):
             return
 
         try:
-            if not self._network_provisioned(tenant_id, network_id):
-                # If we do not have network associated with this, ignore it
-                return
             hostname = self._host_name(host)
             self.rpc.unplug_port_from_network(device_id, device_owner,
                                               hostname, port_id, network_id,
@@ -853,10 +905,10 @@ class AristaDriver(driver_api.MechanismDriver):
             return
         for binding_level in context._binding_levels:
             LOG.debug("deleting segment %s", binding_level.segment_id)
-            if self._network_provisioned(tenant_id, network_id,
+            if self._network_provisioned(context, tenant_id, network_id,
                                          segment_id=binding_level.segment_id):
                 segment = self.ndb.get_segment_by_id(
-                    context._plugin_context.session, binding_level.segment_id)
+                    context._plugin_context, binding_level.segment_id)
                 if not segment:
                     # The segment is already released. Delete it from EOS
                     LOG.debug("Deleting segment %s", binding_level.segment_id)
@@ -884,15 +936,11 @@ class AristaDriver(driver_api.MechanismDriver):
         If this port is the last port using the segmentation id allocated
         by the driver, it should be released
         """
-        host = context.original_host if migration else context.host
-
-        physnet_info = self.eapi.get_physical_network(host)
-        physnet = physnet_info.get('physnet')
-        if not physnet:
-            return
-
-        binding_levels = context.binding_levels
-        LOG.debug("_try_release_dynamic_segment: "
+        if migration:
+            binding_levels = context.original_binding_levels
+        else:
+            binding_levels = context.binding_levels
+        LOG.debug("_try_to_release_dynamic_segment: "
                   "binding_levels=%(bl)s", {'bl': binding_levels})
         if not binding_levels:
             return
@@ -904,7 +952,6 @@ class AristaDriver(driver_api.MechanismDriver):
             driver = binding_level.get(driver_api.BOUND_DRIVER)
             bound_drivers.append(driver)
             if (bound_segment and
-                bound_segment.get('physical_network') == physnet and
                     bound_segment.get('network_type') == p_const.TYPE_VLAN):
                 segment_id = bound_segment.get('id')
                 break
@@ -916,8 +963,7 @@ class AristaDriver(driver_api.MechanismDriver):
         # second last driver in the bound drivers list.
         if (segment_id and bound_drivers[-2:-1] == [MECHANISM_DRV_NAME]):
             filters = {'segment_id': segment_id}
-            result = db_lib.get_port_binding_level(
-                context._plugin_context.session, filters)
+            result = db_lib.get_port_binding_level(filters)
             LOG.debug("Looking for entry with filters=%(filters)s "
                       "result=%(result)s ", {'filters': filters,
                                              'result': result})
@@ -951,7 +997,10 @@ class AristaDriver(driver_api.MechanismDriver):
 
     def _synchronization_thread(self):
         with self.eos_sync_lock:
-            self.sync_service.do_synchronize()
+            try:
+                self.sync_service.do_synchronize()
+            except Exception as e:
+                LOG.warning(e)
 
         self.timer = threading.Timer(self.sync_timeout,
                                      self._synchronization_thread)
@@ -977,7 +1026,7 @@ class AristaDriver(driver_api.MechanismDriver):
             db_lib.forget_network_segment(tenant_network['tenantId'], net_id)
             db_lib.forget_all_ports_for_network(net_id)
 
-    def _network_provisioned(self, tenant_id, network_id,
+    def _network_provisioned(self, context, tenant_id, network_id,
                              segmentation_id=None, segment_id=None):
         # If network does not exist under this tenant,
         # it may be a shared network.
@@ -985,7 +1034,10 @@ class AristaDriver(driver_api.MechanismDriver):
         return (
             db_lib.is_network_provisioned(tenant_id, network_id,
                                           segmentation_id, segment_id) or
-            self.ndb.get_shared_network_owner_id(network_id)
+            self.ndb.get_shared_network_owner_id(
+                context._plugin_context,
+                network_id
+            )
         )
 
     def create_security_group(self, sg):
