@@ -14,10 +14,10 @@
 # limitations under the License.
 
 import mock
+from neutron_lib.api.definitions import portbindings
 from neutron_lib import constants as n_const
 
 from neutron.common import constants as neutron_const
-from neutron.extensions import portbindings
 from neutron.plugins.ml2 import driver_api as api
 from neutron.tests.unit import testlib_api
 
@@ -378,7 +378,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.NeutronNets(),
-            mock.call.__nonzero__(),
             mock.call.is_port_provisioned(port_id),
             mock.call.is_network_provisioned(tenant_id, network_id, None),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
@@ -456,7 +455,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         port_id = port_context.current['id']
         expected_calls = [
-            mock.call.is_port_provisioned(port_id),
+            mock.call.is_port_provisioned(port_id, port_context.host),
             mock.call.forget_port(port_id, port_context.host),
         ]
 
@@ -485,7 +484,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         port_id = port_context.current['id']
         expected_calls += [
-            mock.call.is_port_provisioned(port_id),
+            mock.call.is_port_provisioned(port_id, port_context.host),
             mock.call.forget_port(port_id, port_context.host),
         ]
 
@@ -525,10 +524,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.NeutronNets(),
-            mock.call.__nonzero__(),
-            mock.call.get_physical_network(host_id),
-            mock.call.is_network_provisioned(tenant_id, network_id, None,
-                                             None),
             mock.call.unplug_port_from_network(device_id, 'compute', host_id,
                                                port_id, network_id, tenant_id,
                                                None, vnic_type,
@@ -575,9 +570,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.delete_port_postcommit(port_context)
 
         expected_calls += [
-            mock.call.get_physical_network(host_id),
-            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
-                                             None, None),
             mock.call.unplug_port_from_network(device_id, 'compute', host_id,
                                                port_id, network_id,
                                                INTERNAL_TENANT_ID, None,
@@ -598,7 +590,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
     def test_update_port_precommit(self):
-
         # Test the case where the port was not provisioned previsouly
         # If port is not provisioned, we should bail out
         mechanism_arista.db_lib.is_port_provisioned.return_value = False
@@ -619,7 +610,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                               vm_id,
                                               network_context)
         host_id = port_context.current['binding:host_id']
-        port_context.original['binding:host_id'] = 'ubuntu0'
         port_id = port_context.current['id']
 
         network = {'tenant_id': tenant_id}
@@ -633,7 +623,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.NeutronNets(),
-            mock.call.__nonzero__(),
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id,
                                              segment_id),
@@ -661,7 +650,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                                               vm_id,
                                               network_context)
         host_id = port_context.current['binding:host_id']
-        port_context.original['binding:host_id'] = 'ubuntu0'
         port_id = port_context.current['id']
 
         # Force the check to return port found, but, network was not found
@@ -682,8 +670,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id, segment_id),
             mock.call.is_port_provisioned(port_id, None),
-            mock.call.update_port(vm_id, host_id, port_id,
-                                  network_id, tenant_id)
         ]
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
@@ -706,7 +692,6 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         # Port does not contain a tenant
         port_context.current['tenant_id'] = None
         host_id = port_context.current['binding:host_id']
-        port_context.original['binding:host_id'] = 'ubuntu0'
         port_id = port_context.current['id']
 
         # Force the check to return port and network were found
@@ -759,6 +744,88 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         ]
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
+    def test_update_port_precommit_migration_case(self):
+        mechanism_arista.db_lib.is_port_provisioned.return_value = True
+
+        tenant_id = 'ten-1'
+        network_id = 'net1-id'
+        vm_id = 'vm1'
+
+        network_context = self._get_network_context_hpb(tenant_id,
+                                                        network_id)
+
+        port_context = self._get_port_context_hpb(tenant_id,
+                                                  network_id,
+                                                  vm_id,
+                                                  network_context,
+                                                  migration=True)
+        host_id = port_context.current['binding:host_id']
+        port_id = port_context.current['id']
+
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+
+        self.drv.update_port_precommit(port_context)
+
+        expected_calls = [
+            mock.call.NeutronNets(),
+            mock.call.is_port_provisioned(port_id, port_context.original_host),
+            mock.call.update_port(vm_id, host_id, port_id,
+                                  network_id, tenant_id)
+        ]
+
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
+    def test_update_port_postcommit_migration_case(self):
+        tenant_id = 'ten-1'
+        network_id = 'net1-id'
+        vm_id = 'vm1'
+
+        network_context = self._get_network_context_hpb(tenant_id,
+                                                        network_id)
+        segments = network_context.network_segments
+        port_context = self._get_port_context_hpb(tenant_id,
+                                                  network_id,
+                                                  vm_id,
+                                                  network_context,
+                                                  migration=True)
+
+        mechanism_arista.db_lib.is_network_provisioned.return_value = True
+        mechanism_arista.db_lib.num_nets_provisioned.return_value = 1
+        mechanism_arista.db_lib.num_vms_provisioned.return_value = 1
+        mechanism_arista.db_lib.get_port_binding_level.return_value = None
+
+        network = {'tenant_id': tenant_id}
+        self.drv.ndb.get_network_from_net_id.return_value = [network]
+        port = port_context.original
+        device_id = port['device_id']
+        device_owner = port['device_owner']
+        orig_host_id = port_context.original_host
+        port_id = port['id']
+        vnic_type = port['binding:vnic_type']
+        profile = port['binding:profile']
+        sg = port['security_groups']
+
+        self.drv.update_port_postcommit(port_context)
+
+        filters = {'segment_id': segments[1]['id']}
+        expected_calls = [
+            mock.call.NeutronNets(),
+            mock.call.get_port_binding_level(filters),
+            mock.call.unplug_port_from_network(device_id, device_owner,
+                                               orig_host_id, port_id,
+                                               network_id, tenant_id,
+                                               sg, vnic_type,
+                                               switch_bindings=profile),
+            mock.call.remove_security_group(None, []),
+            mock.call.num_nets_provisioned(tenant_id),
+            mock.call.num_vms_provisioned(tenant_id),
+        ]
+        expected_calls.extend([mock.call.is_network_provisioned(
+            tenant_id, network_id, None, segment['id'])
+            for segment in segments])
+        mechanism_arista.db_lib.assert_has_calls(expected_calls)
+
     def test_update_port_postcommit(self):
         tenant_id = 'ten-1'
         network_id = 'net1-id'
@@ -788,8 +855,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         device_id = port['device_id']
         device_owner = port['device_owner']
         host_id = port['binding:host_id']
-        orig_host_id = 'ubuntu0'
-        port_context.original['binding:host_id'] = orig_host_id
+        orig_host_id = port_context.original_host
         port_id = port['id']
         port_name = port['name']
         vnic_type = port['binding:vnic_type']
@@ -801,24 +867,15 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.NeutronNets(),
-            mock.call.__nonzero__(),
             mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None, None),
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id, None),
             mock.call.hpb_supported(),
             mock.call.create_network_segments(tenant_id, network_id,
                                               network_name,
                                               segments),
-            mock.call.is_network_provisioned(tenant_id, network_id, None,
-                                             None),
-            mock.call.unplug_port_from_network(device_id, 'compute',
-                                               orig_host_id, port_id,
-                                               network_id, tenant_id,
-                                               None, vnic_type,
-                                               switch_bindings=profile),
-            mock.call.remove_security_group(None, profile),
-            mock.call.num_nets_provisioned(tenant_id),
-            mock.call.num_vms_provisioned(tenant_id),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
                                              network_id, tenant_id,
                                              port_name, device_owner, None,
@@ -861,8 +918,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         device_id = port['device_id']
         device_owner = port['device_owner']
         host_id = port['binding:host_id']
-        orig_host_id = 'ubuntu0'
-        port_context.original['binding:host_id'] = orig_host_id
+        orig_host_id = port_context.original_host
         port_id = port['id']
         port_name = port['name']
         network_name = network_context.current['name']
@@ -871,23 +927,14 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls += [
             mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
+                                             None, None),
             mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
                                              segmentation_id, None),
             mock.call.hpb_supported(),
             mock.call.create_network_segments(INTERNAL_TENANT_ID, network_id,
                                               network_name,
                                               segments),
-            mock.call.is_network_provisioned(INTERNAL_TENANT_ID, network_id,
-                                             None, None),
-            mock.call.unplug_port_from_network(device_id, 'compute',
-                                               orig_host_id,
-                                               port_id, network_id,
-                                               INTERNAL_TENANT_ID,
-                                               None, vnic_type,
-                                               switch_bindings=profile),
-            mock.call.remove_security_group(None, profile),
-            mock.call.num_nets_provisioned(INTERNAL_TENANT_ID),
-            mock.call.num_vms_provisioned(INTERNAL_TENANT_ID),
             mock.call.plug_port_into_network(device_id, host_id, port_id,
                                              network_id, INTERNAL_TENANT_ID,
                                              port_name, device_owner, None,
@@ -940,6 +987,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls += [
             mock.call.is_port_provisioned(port_id, port_context.host),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None, None),
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id, None),
             mock.call.hpb_supported(),
@@ -963,14 +1012,14 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls += [
             mock.call.is_port_provisioned(port_id, port_context.host),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None, None),
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id, None),
             mock.call.hpb_supported(),
             mock.call.create_network_segments(tenant_id, network_id,
                                               network_name,
                                               segments),
-            mock.call.is_network_provisioned(tenant_id, network_id,
-                                             None, None),
             mock.call.unplug_port_from_network(device_id, owner,
                                                orig_host_id,
                                                port_id, network_id,
@@ -1067,10 +1116,7 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         self.drv.update_port_precommit(context)
 
         expected_calls = [
-            mock.call.is_network_provisioned(tenant_id, network_id,
-                                             segmentation_id,
-                                             segment_id),
-            mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_port_provisioned(port_id, old_host),
             mock.call.update_port(new_device_id,
                                   new_host, port_id,
                                   network_id, tenant_id)
@@ -1205,12 +1251,12 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None, None),
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id,
                                              None),
             mock.call.hpb_supported(),
-            mock.call.is_network_provisioned(tenant_id, network_id,
-                                             None, None),
             mock.call.unplug_port_from_network(old_device_id,
                                                n_const.DEVICE_OWNER_DHCP,
                                                old_host,
@@ -1234,17 +1280,14 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
         context.current['binding:host_id'] = new_host
         context.current['status'] = 'DOWN'
 
+        physnet = dict(physnet='default')
+        self.fake_rpc.get_physical_network.return_value = physnet
+        context._original_binding_levels = context._binding_levels
+
         mechanism_arista.db_lib.reset_mock()
         self.drv.update_port_postcommit(context)
 
         expected_calls = [
-            mock.call.is_port_provisioned(port_id, None),
-            mock.call.is_network_provisioned(tenant_id, network_id,
-                                             segmentation_id,
-                                             None),
-            mock.call.hpb_supported(),
-            mock.call.is_network_provisioned(tenant_id, network_id,
-                                             None, None),
             mock.call.unplug_port_from_network(reserved_device,
                                                n_const.DEVICE_OWNER_DHCP,
                                                old_host,
@@ -1255,15 +1298,11 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
             mock.call.remove_security_group(None, profile),
             mock.call.num_nets_provisioned(tenant_id),
             mock.call.num_vms_provisioned(tenant_id),
-            mock.call.plug_port_into_network(new_device_id,
-                                             new_host,
-                                             port_id, network_id,
-                                             tenant_id, port_name,
-                                             n_const.DEVICE_OWNER_DHCP,
-                                             None, None, vnic_type,
-                                             segments=[],
-                                             switch_bindings=profile),
         ]
+        expected_calls.extend(
+            mock.call.is_network_provisioned(tenant_id, network_id, None,
+                                             binding_level.segment_id)
+            for binding_level in context._original_binding_levels)
 
         mechanism_arista.db_lib.assert_has_calls(expected_calls)
 
@@ -1280,6 +1319,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None, None),
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id,
                                              None),
@@ -1310,6 +1351,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None, None),
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id,
                                              None),
@@ -1340,6 +1383,8 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
 
         expected_calls = [
             mock.call.is_port_provisioned(port_id, None),
+            mock.call.is_network_provisioned(tenant_id, network_id,
+                                             None, None),
             mock.call.is_network_provisioned(tenant_id, network_id,
                                              segmentation_id,
                                              None),
@@ -1389,11 +1434,62 @@ class AristaDriverTestCase(testlib_api.SqlTestCase):
                      }
         binding_levels = []
         for level, segment in enumerate(network.network_segments):
-            binding_levels.append(FakePortBindingLevel(port['id'],
-                                                       level,
-                                                       'vendor-1',
-                                                       segment['id']))
-        return FakePortContext(port, dict(orig_port), network, status,
+            binding_levels.append(FakePortBindingLevel(
+                port['id'], level, 'vendor-1',
+                segment['id']))
+        return FakePortContext(port, dict(orig_port), network, binding_levels)
+
+    def _get_network_context_hpb(self, tenant_id, net_id):
+        network = {'id': net_id,
+                   'tenant_id': tenant_id,
+                   'name': 'test-net'}
+        segmentation_id = 1001
+        vni = segmentation_id * 10
+        network_segments = [{'segmentation_id': vni,
+                             'physical_network': None,
+                             'id': 'segment-id-for-%s' % vni,
+                             'network_type': 'vxlan'},
+                            {'segmentation_id': segmentation_id,
+                             'physical_network': u'switch1',
+                             'id': 'segment-id-for-%s' % segmentation_id,
+                             'network_type': 'vlan'}]
+        return FakeNetworkContext(tenant_id, network, network_segments,
+                                  network)
+
+    def _get_port_context_hpb(self, tenant_id, net_id, device_id, network,
+                              device_owner='compute', migration=False):
+        host_id = 'ubuntu1'
+        port = {'device_id': device_id,
+                'device_owner': device_owner,
+                'binding:host_id': '' if migration else host_id,
+                'name': 'test-port',
+                'tenant_id': tenant_id,
+                'id': 101,
+                'network_id': net_id,
+                'binding:vnic_type': None,
+                'binding:profile': [],
+                'security_groups': None,
+                'status': 'DOWN' if migration else 'ACTIVE'
+                }
+        orig_port = {'device_id': device_id,
+                     'device_owner': device_owner,
+                     'binding:host_id': 'ubuntu1',
+                     'name': 'test-port',
+                     'tenant_id': tenant_id,
+                     'id': 101,
+                     'network_id': net_id,
+                     'binding:vnic_type': 'baremetal',
+                     'binding:profile': [],
+                     'security_groups': None,
+                     'status': 'ACTIVE'
+                     }
+        binding_levels = []
+        bound_driver = ['arista', 'vendor-1']
+        for level, segment in enumerate(network.network_segments):
+            binding_levels.append(FakePortBindingLevel(
+                port['id'], level, bound_driver[level],
+                segment['id']))
+        return FakePortContext(port, dict(orig_port), network, None,
                                binding_levels)
 
 
@@ -1436,15 +1532,15 @@ class FakeNetworkContext(object):
 class FakePortContext(object):
     """To generate port context for testing purposes only."""
 
-    def __init__(self, port, original_port, network, status,
-                 binding_levels):
+    def __init__(self, port, original_port, network,
+                 binding_levels, orig_binding_levels=None):
         self._plugin_context = FakePluginContext('test')
         self._port = port
         self._original_port = original_port
         self._network_context = network
-        self._status = status
+        self._status = port['status']
         self._binding_levels = binding_levels
-        self._original_binding_levels = []
+        self._original_binding_levels = orig_binding_levels
 
     @property
     def current(self):
@@ -1483,6 +1579,14 @@ class FakePortContext(object):
                 api.BOUND_SEGMENT: self._expand_segment(level.segment_id)
             } for level in self._binding_levels]
 
+    @property
+    def original_binding_levels(self):
+        if self._original_binding_levels:
+            return [{
+                api.BOUND_DRIVER: level.driver,
+                api.BOUND_SEGMENT: self._expand_segment(level.segment_id)
+            } for level in self._original_binding_levels]
+
     @property
     def bottom_bound_segment(self):
         if self._binding_levels:
@@ -1493,6 +1597,9 @@ class FakePortContext(object):
             if segment[api.ID] == segment_id:
                 return segment
 
+    def release_dynamic_segment(self, segment_id):
+        pass
+
 
 class FakePluginContext(object):
     """Plugin context for testing purposes only."""
